/*
 * libslb.h - internal header file.
 * Used only when compiling the shared library code,
 * to redirect standard library functions to the
 * library that was linked to the application
 *
 * Copyright (C) 2018 Thorsten Otto
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __ICONVLIBSLB_H__
#define __ICONVLIBSLB_H__ 1

#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <wchar.h>

#undef errno

#undef memset
#undef memcpy
#undef memcmp

#undef strlen
#undef strcpy
#undef strcmp
#undef strchr

#undef malloc
#undef free

#undef getenv
#undef qsort

#undef wcrtomb
#undef mbrtowc
#undef mbsinit

#include "libstrct.h"

struct _iconvslb_funcs *libiconv_get_slb_funcs(void);

#define memset(d, c, l) libiconv_get_slb_funcs()->p_memset(d, c, l)
#define memcpy(d, s, l) libiconv_get_slb_funcs()->p_memcpy(d, s, l)
#define memcmp(d, s, l) libiconv_get_slb_funcs()->p_memcmp(d, s, l)

#define strlen libiconv_get_slb_funcs()->p_strlen
#define strcpy libiconv_get_slb_funcs()->p_strcpy
#define strcmp libiconv_get_slb_funcs()->p_strcmp
#define strchr libiconv_get_slb_funcs()->p_strchr

#define malloc libiconv_get_slb_funcs()->p_malloc
#define free libiconv_get_slb_funcs()->p_free

#define errno (libiconv_get_slb_funcs()->p_get_errno())
#undef __set_errno
#define __set_errno(e) (libiconv_get_slb_funcs()->p_set_errno(e))
#define getenv libiconv_get_slb_funcs()->p_getenv
#define qsort libiconv_get_slb_funcs()->p_qsort

#define wcrtomb libiconv_get_slb_funcs()->p_wcrtomb
#define mbrtowc libiconv_get_slb_funcs()->p_mbrtowc
#define mbsinit libiconv_get_slb_funcs()->p_mbsinit

#endif /* __ICONVLIBSLB_H__ */
